/*
正体補正プログラム
超音波センサの番号は、次のようになっていることを前提としている。
	USS1:正面左側     USS2:正面右側

*/

#include <linux/errno.h>
#include <sys/io.h>
#include <unistd.h>
#include <math.h>
#include <time.h>
#include <pthread.h>
#include <stdio.h>
#include <stdlib.h>
//#include "hardware.h"
#include "serial_ctrl.h"
#include "motor.h"
#include "direction.h"


#define COUNT_MAX 10		// 正体補正を諦めるまでの回数
#define USSMAX	  254		//所定距離の最大値[cm]
#define USSMIN	  3		//所定距離の最小値[cm]
#define SLEEP	  0.1	//秒単位
#define TOLER     3		//正対補正許容範囲
#define USS_DIST  21	//超音波センサ間の距離
#define PWMS      20
#define PWMR      20
#define TOLER_R   5
static float ref_angle;
static int direction_endflag = 0;

/* 引数 dist：正体補正する物体までの距離 */
int direction_correct(int dist){
	direction_endflag = 0;

	int uss_l;			//left
	int uss_r;			//right
	int diff_uss;
	float diff_max=10;
	int c_count=0;
	int cur_dist;		//現在の前方との距離
	int ref_dist = 0;	//目標値
	int deviation = 0;
  int pwmr = 28;
	//float resol;

	printf("Direction Correction Start !!1\n") ;

	//resol = 1. / USS_DIST * 180.0 /3.1415; // [deg]
	//printf("angle resoution =%8.3f\n",resol) ;

// 距離の補正
	if(dist != 0){

		printf("距離補正スタート\n");
		ref_dist = dist + 10;
		uss_l=uss_distance[2]; // left
		usleep(5);
		uss_r=uss_distance[3]; // right
		usleep(5);

		if(cur_dist!=999){
			//許容範囲までループ
			while (1){   				
				uss_l = uss_distance[2];
				usleep(5);
				uss_r = uss_distance[3];
				usleep(5);
				printf("LEFT:::%d  RIGHT:::%d\n", uss_l, uss_r);

				//両方のUSSの距離を信頼する場合
				if ((uss_l > USSMIN && uss_l < USSMAX) && (uss_r > USSMIN && uss_r < USSMAX)){
					cur_dist = (uss_l + uss_r) / 2;
				} // 左のUSSの距離を信頼する場合
				else if (uss_l > USSMIN && uss_l < USSMAX){
					cur_dist = uss_l;
				} // 右のUSSの距離を信頼する場合
				else if (uss_r > USSMIN && uss_r < USSMAX){
					cur_dist = uss_r;
				}
				else{ //両方とも信頼出来ない場合
					cur_dist = 999;
				}
				deviation = ref_dist - cur_dist;
        printf("LEFT:::%d RIGHT::: %d\n",uss_l,uss_r);
        printf("目標値:::%d   現在:::%d   差分:::%d", ref_dist, cur_dist, deviation);

        if((deviation>= -TOLER) && (deviation <= TOLER)){
            motor_drive(0,0);
            break;
        }
				if (deviation < 0){
					motor_drive(PWMS-1.5,PWMS+1.5);		//直進動作を初期プログラムはし
				}else{
					motor_drive(-PWMS+2,-PWMS);
				}
			}
		}
    usleep(1000*1000); //1秒待つ
	printf("直線補正完了\n");
	}


// 正体補正
	

	printf("正体補正スタート\n");

	while(1){
		uss_l = uss_distance[2]; // left
		usleep(5);
		uss_r = uss_distance[3]; // right
		usleep(5);
		printf("uss_l=%d   uss_r= %d\n",uss_l,uss_r);
/*
      while((uss_l == 255) || (uss_r == 255)){
      uss_l = uss_distance[2];
      uss_r + uss_distance[3];
      printf("USS測定エラー\n");
      }
*/
    diff_uss = uss_l - uss_r;
    printf("差分:::%d\n",diff_uss);
    
		if ((diff_uss >= -TOLER) && (diff_uss <= TOLER)){
      motor_drive(0,0);
      pwmr = 35;
      int end = 0;
      int i;
      int uss_data2 = 0;
      int uss_data3 = 0;
      for(i = 0;i<10;i++){
        uss_data2 += uss_distance[2];
        uss_data3 += uss_distance[3];
        usleep(200);
        if((i == 9) && abs(uss_data2 - uss_data3) <=TOLER_R){
          printf("***complit***\n");
          end = 1;
        }
       }
      if(end == 1){
        sleep(1);
        break;
      }
    }
		float deg = acos(USS_DIST / sqrt(USS_DIST ^ 2 + diff_uss ^ 2));  //	回転角度算出

    if(uss_l > uss_r){
      motor_drive(pwmr, -pwmr);
    }
    else{
      motor_drive(-pwmr,pwmr);
    }
    usleep(20);
	}

	/*
		if(uss_l > uss_r){
			int cur_deg = (int)(asin(uss_l - uss_r) / (sqrt((uss_l - uss_r)^2 + ^2)));
			int ref_deg  = compass_angle + cur_deg;
			int deg;
			while(1){
				deg = compass_angle;
				printf("右回転　%d°",cur_deg);
				motor_drive(55,-55);
				usleep(200);
				if((ref_deg + 3 > deg) && (ref_deg - 3 < deg)){
					break;
				}
			}			
		}
		else{
			int deg = (int)(asin(uss_r - uss_l) / (sqrt((uss_r - uss_l)^2 + ^2)));
			int ref_deg  = compass_angle + cur_deg;
			int deg;
			while(1){
				deg = compass_angle;
				printf("左回転　%d°",cur_deg);
				motor_drive(-55,55);
				usleep(200);
				if((ref_deg + 3 > deg) && (ref_deg - 3 < deg)){
					break;
				}
			}
		}
	*/
  while(1){
    uss_l = uss_distance[2];
    usleep(5);
    uss_r = uss_distance[3];
    usleep(5);
    cur_dist = (uss_l + uss_r) / 2;
    deviation = ref_dist - cur_dist - 3;
    if((deviation <= TOLER) && (deviation >= -TOLER)){
    motor_drive(0,0);
    break;
    }
    if((ref_dist - cur_dist) < 0){
      motor_drive(PWMS,PWMS);
    }
    else{
      motor_drive(-PWMS,-PWMS);
    }
   }
  printf("OSHIMAI\n");
	
	direction_endflag = 1;
//return 0;
}

int direction_endflag_get(){
	return direction_endflag;
}

